/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.common.elasticsearch;

import java.util.Base64;

import org.apache.commons.collections.MapUtils;
import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.elasticsearch.client.RestClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.json.jackson.JacksonJsonpMapper;
import co.elastic.clients.transport.rest_client.RestClientTransport;
import de.ozgcloud.operator.OzgCloudElasticsearchProperties;
import de.ozgcloud.operator.common.kubernetes.KubernetesRemoteService;
import io.fabric8.kubernetes.api.model.Secret;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Configuration
public class ElasticsearchClientConfiguration {

	@Autowired
	private KubernetesRemoteService kubernetesService;
	@Autowired
	private OzgCloudElasticsearchProperties elasticSearchProperties;

	@Bean
	@Scope("singleton")
	ElasticsearchClient createElasticsearchClient() {
		LOG.info("Create elasticsearch client...");
		var credentialsProvider = createCredentialsProvider(elasticSearchProperties.getServer().getSecretDataKey(), getPassword());
		var restClient = buildRestClient(credentialsProvider);
		var transport = createRestClientTransport(restClient);
		return new ElasticsearchClient(transport);
	}

	private BasicCredentialsProvider createCredentialsProvider(String userName, String password) {
		var credentialsProvider = new BasicCredentialsProvider();
		credentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(userName, password));
		return credentialsProvider;
	}

	private RestClient buildRestClient(BasicCredentialsProvider credentialsProvider) {
		return RestClient.builder(createHttpHost())
				.setHttpClientConfigCallback(httpClientBuilder -> httpClientBuilder.setDefaultCredentialsProvider(credentialsProvider))
				.build();
	}

	private HttpHost createHttpHost() {
		LOG.info(String.format("ElasticSearch config: host: %s with port: %s and scheme: %s", elasticSearchProperties.getServer().getHost(),
				elasticSearchProperties.getServer().getPort(), elasticSearchProperties.getServer().getScheme()));
		return new HttpHost(elasticSearchProperties.getServer().getHost(), elasticSearchProperties.getServer().getPort(),
				elasticSearchProperties.getServer().getScheme());
	}

	private RestClientTransport createRestClientTransport(RestClient restClient) {
		return new RestClientTransport(restClient, new JacksonJsonpMapper());
	}

	String getPassword() {
		LOG.debug(String.format("get password from secret: %s in namespace %s", elasticSearchProperties.getServer().getSecretName(),
				elasticSearchProperties.getServer().getNamespace()));
		var secret = getCredentialsSecret();
		var password = getPasswordFromSecret(secret);
		return password;
	}

	private Secret getCredentialsSecret() {
		return kubernetesService
				.getSecretResource(elasticSearchProperties.getServer().getNamespace(), elasticSearchProperties.getServer().getSecretName()).get();
	}

	private String getPasswordFromSecret(Secret secret) {
		var encodedPassword = MapUtils.getString(secret.getData(), elasticSearchProperties.getServer().getSecretDataKey());
		return decode(encodedPassword, secret);
	}

	private String decode(String encodedPassword, Secret secret) {
		return new String(Base64.getDecoder().decode(encodedPassword));
	}

}