/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Base64;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.operator.common.kubernetes.NamespaceTestFactory;
import de.ozgcloud.operator.common.kubernetes.SecretTestFactory;
import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.api.model.SecretBuilder;
import io.javaoperatorsdk.operator.api.reconciler.Context;
import io.javaoperatorsdk.operator.api.reconciler.UpdateControl;
import lombok.SneakyThrows;

class ElasticsearchReconcilerTest {

	@Spy
	@InjectMocks
	private ElasticsearchReconciler reconciler;
	@Mock
	private OzgCloudElasticsearchService service;

	@DisplayName("Reconcile")
	@Nested
	class TestReconcile {

		@Mock
		private Context<OzgCloudElasticsearchCustomResource> context;

		private final OzgCloudElasticsearchCustomResource resource = ElasticsearchCustomResourceTestFactory.create();

		private final static String PASSWORD = new String(Base64.getEncoder().encode("dummyPassword".getBytes()));
		private final static String DECODED_PASSWORD = new String(Base64.getDecoder().decode(PASSWORD));
		private final Secret secret = SecretTestFactory.createBuilder()
				.addToData(OzgCloudElasticsearchSecretHelper.CREDENTIAL_SECRET_PASSWORD_FIELD, PASSWORD).build();

		@DisplayName("process flow")
		@Nested
		class TestProcessFlow {

			@BeforeEach
			void mockCredentialSecret() {
				when(service.getOrCreateCredentialSecret(any(), any())).thenReturn(secret);
			}

			@Test
			void shouldGetCredentialSecret() {
				reconcile();

				verify(service).getOrCreateCredentialSecret(resource, context);
			}

			@SneakyThrows
			@Test
			void shouldCreateIndexIfMissing() {
				reconcile();

				verify(service).createIndexIfMissing(NamespaceTestFactory.NAMESPACE);
			}

			@SneakyThrows
			@Test
			void shouldCreateecurityRoleIfMissing() {
				reconcile();

				verify(service).createSecurityRoleIfMissing(NamespaceTestFactory.NAMESPACE);
			}

			@SneakyThrows
			@Test
			void shouldCallCreateSecurityUserIfMissing() {
				reconcile();

				verify(service).createSecurityUserIfMissing(NamespaceTestFactory.NAMESPACE, DECODED_PASSWORD);
			}

			@SneakyThrows
			@Test
			void shouldCallCreateCertificateIfMissing() {
				reconcile();

				verify(service).createCertificateIfMissing(NamespaceTestFactory.NAMESPACE);
			}
		}

		@DisplayName("on exception")
		@Nested
		class TestOnException {

			private final Exception exception = new RuntimeException();

			@BeforeEach
			void mock() {
				when(service.getOrCreateCredentialSecret(any(), any())).thenThrow(exception);
			}

			@Test
			void shouldBuildExceptionUpdateControl() {
				reconcile();

				verify(reconciler).buildExceptionUpdateControl(resource, exception);
			}
		}

		@SneakyThrows
		private UpdateControl<OzgCloudElasticsearchCustomResource> reconcile() {
			return reconciler.reconcile(resource, context);
		}

		@DisplayName("build exception update control")
		@Nested
		class TestBuildExceptionUpdateControl {

			private final OzgCloudElasticsearchCustomResource resource = ElasticsearchCustomResourceTestFactory.create();
			private final static String EXCEPTION_MESSAGE = "ExeptionMessage";
			private final Exception exception = new RuntimeException(EXCEPTION_MESSAGE);

			@Test
			void shouldContainResource() {
				var updateControl = buildExceptionUpdateControl();

				assertThat(updateControl.getResource()).isEqualTo(resource);
			}

			@Test
			void shouldContainUpdateStatus() {
				var updateControl = buildExceptionUpdateControl();

				assertThat(updateControl.getResource().getStatus().getStatus()).isEqualTo(CustomResourceStatus.ERROR);
			}

			@Test
			void shouldContainReschedule() {
				var updateControl = buildExceptionUpdateControl();

				assertThat(updateControl.getScheduleDelay()).hasValue(60000L);
			}

			@Test
			void shouldContainMessage() {
				var updateControl = buildExceptionUpdateControl();

				assertThat(updateControl.getResource().getStatus().getMessage()).isEqualTo(EXCEPTION_MESSAGE);
			}

			private UpdateControl<OzgCloudElasticsearchCustomResource> buildExceptionUpdateControl() {
				return reconciler.buildExceptionUpdateControl(resource, exception);
			}
		}

		@DisplayName("get password")
		@Nested
		class TestGetPassword {

			private static final String SECRET_PASSWORD = LoremIpsum.getInstance().getWords(1);

			@Test
			void shouldReturnPasssowrd() {
				var secret = buildSecret();

				var password = reconciler.getPassword(secret);

				assertThat(password).isEqualTo(SECRET_PASSWORD);
			}

			private Secret buildSecret() {
				return new SecretBuilder()
						.addToData(Map.of(OzgCloudElasticsearchSecretHelper.CREDENTIAL_SECRET_PASSWORD_FIELD,
								encodeStringBase64(SECRET_PASSWORD)))
						.build();
			}

			private String encodeStringBase64(String string) {
				return Base64.getEncoder().encodeToString(string.getBytes());
			}
		}
	}

	@DisplayName("Cleanup")
	@Nested
	class TestCleanup {

		@Mock
		private Context<OzgCloudElasticsearchCustomResource> context;

		private final OzgCloudElasticsearchCustomResource resource = ElasticsearchCustomResourceTestFactory.create();

		@SneakyThrows
		@Test
		void shouldDeleteSecurityUser() {
			reconciler.cleanup(resource, context);

			verify(service).deleteSecurityUserIfExists(NamespaceTestFactory.NAMESPACE);
		}

		@SneakyThrows
		@Test
		void shouldDeleteSecurityRole() {
			reconciler.cleanup(resource, context);

			verify(service).deleteSecurityRoleIfExists(NamespaceTestFactory.NAMESPACE);
		}

		@SneakyThrows
		@Test
		void shouldDeleteIndex() {
			reconciler.cleanup(resource, context);

			verify(service).deleteIndexIfExists(NamespaceTestFactory.NAMESPACE);
		}

		@Test
		void shouldReturnDeleteControl() {
			var deleteControl = reconciler.cleanup(resource, context);

			assertThat(deleteControl).isNotNull();
		}
	}
}